***************************** EQU TABLE *****************************
* The following three values are application dependent
BASE    EQU     $0700000        ; This is set according to the value in BAR
INIT    EQU     $0030300        ; Initialization Routine
INT_VEC EQU     $0031000        ; Interrupt Vector for SCC1
LOOKUP  EQU     $0030200        ; Look up table for baud rate
LK_SZ   EQU     15      ; Number of entries in the lookup table
RXCHARS EQU     $4000   ; Pointer to receive buffer for characters
NOMSP   EQU     BASE+$4A0       ; Nominal speed reported for SCC1
RXINTPT EQU     BASE+$488       ; Pointer to internal receive pointer
TXINTPT EQU     BASE+$494       ; Pointer to internal transmit pointer
CARRT   EQU     $0F     ; ASCII for Carriage Return

* Commands
RSTCP   EQU     $81     ; Reset CP
STPTX1  EQU     $01     ; STOP TRANSMIT SCC1
RSTX1   EQU     $11     ; RESTART TRANSMIT SCC1
ENHM1   EQU     $21     ; ENTER HUNT MODE SCC1
ENBHM1  EQU     $071    ; ENTER BAUD HUNT MODE SCC1

CR      EQU     BASE+$0860      ; Command Register
BAR     EQU     $0F2    ; Base Address Register
MER     EQU     $0F8    ; UCODE Option Enabling Register
SCR     EQU     $0F4    ; System Control Register
GIMR    EQU     BASE+$0812      ; Global Interrupt Mode Register
IPR     EQU     BASE+$0814      ; Interrupt Pending Register
IMR     EQU     BASE+$0816      ; Interrupt Mask Register
ISR     EQU     BASE+$0818      ; In-Service Register
PACNT   EQU     BASE+$081E      ; PORT A CONTROL
PBCNT   EQU     BASE+$0824
SIMODE  EQU     BASE+$08B4      ; Serial Interface Mode Register

SCON1   EQU     BASE+$0882      ; SCC1 Configuration Register
SCM1    EQU     BASE+$0884      ; SCC1 Mode Register
DSR1    EQU     BASE+$0886      ; SCC1 DSR Register
SCCE1   EQU     BASE+$0888      ; SCC1 Event Register
SCCM1   EQU     BASE+$088A      ; SCC1 Mask Register

EN_SCC  EQU     $0C     ; ENR and ENT bits in SCM
ENRON   EQU     $08     ; For Setting ENR
ENTON   EQU     $04     ; For Setting ENT
ENTOFF  EQU     $0FFFB  ; For Clearing ENT

*** SCC Parameter Table ***
ST_BD   EQU     0       ; Status and Control in BD
SS_BD   EQU     1       ; Status in BD
LN_BD   EQU     2       ; Data Length in BD
PT_BD   EQU     4       ; Buffer pointer in BD
SZ_BD   EQU     $08     ; Size of BD = 8 bytes

READY   EQU     $07     ; Ready bit in the 1st byte of TX BD
EMPTY   EQU     $07     ; Empty bit in the 1st byte of RX BD
WRAP    EQU     $05     ; Wrap bit in the 1st byte of BD

* SCC1 Parameter *
FCR_1   EQU     BASE+$0480      ; RFCR and TFCR for SCC1
MRBLR_1 EQU     BASE+$0482      ; Max Rx Buffer Length
MIDL_1  EQU     BASE+$049C      ; Max IDLE char (received)
BRKCR_1 EQU     BASE+$04A0      ; Break counter (transmitted)
PAREC_1 EQU     BASE+$04A2      ; RX Parity error
FRMEC_1 EQU     BASE+$04A4      ; RX Framing error
NOSEC_1 EQU     BASE+$04A6      ; RX Noise counter
BRKEC_1 EQU     BASE+$04A8      ; RX Break condition counter
UADR1_1 EQU     BASE+$04AA      ; UART Address char 1
UADR2_1 EQU     BASE+$04AC      ; UART Address char 2
RCCR_1  EQU     BASE+$04AE      ; RX Control Char
RCH1_1  EQU     BASE+$04B0      ; CONTROL char 1
RCH6_1  EQU     BASE+$04BA      ; L word to hold ptr to char buffer
RCH8_1  EQU     BASE+$04BE      ; CONTROL char 8

* Autobaud special
ABCR1_1 EQU     BASE+$04AA      ; User defined Char 1 (a/A)
ABCR2_1 EQU     BASE+$04AC      ; User defined Char 2 (a/A)
RxPTR   EQU     BASE+$04BA      ; Pointer to external Rx data buffer
TxBD    EQU     BASE+$04BE      ; Transmit character descriptor
RXBD_01 EQU     BASE+$0400      ; 1st RXBD of SCC1
TXBD_01 EQU     BASE+$0440      ; 1st TXBD of SCC1
RXBD_71 EQU     BASE+$0438      ; 7th RXBD of SCC1 for Autobaud command


* The followings are application dependent, for this example
BD_CNT  EQU     $08     ; Number of BDs used
SZ_BF   EQU     $10     ; Size of buffer = 16 bytes
N_DATA  EQU     $0A     ; Number of data to be sent in a buffer

******************************* Data Used ******************************

	ORG     LOOKUP  ;assumes that a 1.8432 MHz clock is on TIN1
	DC.W    17
	DC.W    8       ;115200
	DC.W    34
	DC.W    16      ;57600
	DC.W    50
	DC.W    24      ;38400
	DC.W    67
	DC.W    32      ;28800
	DC.W    101
	DC.W    48      ;19200
	DC.W    134
	DC.W    64      ;14400
	DC.W    161
	DC.W    77      ;12000
	DC.W    202
	DC.W    96      ;9600
	DC.W    269
	DC.W    128     ;7200
	DC.W    403
	DC.W    192     ;4800
	DC.W    806
	DC.W    384     ;2400
	DC.W    1613
	DC.W    768     ;1200
	DC.W    3226
	DC.W    1536    ;600
	DC.W    6451
	DC.W    3072    ;300
	DC.W    17594
	DC.W    8378    ;110
	
RXFLAG  DC.W    0       ;Flag for character reception

******************** M68302 Initialization Code *****************
* Register Initialized values in ADS board before execution
* USP=00080000 ISP=000040000 (Stack pointer not used)
* This program assumes that the AutoBaud microcode package has
* already been downloaded into the internally dual-port RAM.

	ORG     INIT    ; PC=00030300
	MOVE.W  #$2700,SR       ; SR=2700, mask off interrupts
	MOVE.W  #$0700,BAR      ; BAR=0700
	LEA.L   $40000,A7       ; Load initial stack pointer

	MOVE.W  #$0001,MER      ; Enable Ucode for autobaud
	MOVE.B  #RSTCP,CR       ; Reset CR of communication processor
LP00    BTST.B  #0,CR
	BNE.B   LP00

	MOVE.L  #0,SCR  ; Nothing special for this example

*** Setups for interrupt ***
	MOVE.W  #$0A0,GIMR      ; Normal mode, v7-v5=5
	MOVE.W  #0,IMR  ; Mask off all for now
	MOVE.W  #$FFFF,IPR      ; Clear IPR

*** Set up Serial Interface Connection ***
	MOVE.W  #$0008,PBCNT    ; FOR TIN1 INSTEAD OF PB3

* Set up SIMODE = NMSI Normal operation
	MOVE.W  #$0,SIMODE

***************** SCC1 Initialization for Autobaud ********************
* Interrupt Vector: SCC1 interrupt handler is at INTSCC1
* v7-v5=5, v4-v0=$0d => vector=$ad => Exception vector = ($ad<<2) = $2b4
	MOVE.L  #INTSCC1,$02B4

************************* begin autobaud *****************************
	MOVE.W  #$4000,SCON1    ; Set initial divider
	MOVE.W  #$7FFF,DSR1     ; Initialize DSR to look for START bit

* Set SCC1 Mode for autobaud - BISYNC mode, s/w operation, REVD and RBCS set.
	MOVE.W  #$1133,SCM1
***************************** end autobaud ***************************

* Set up Parameter RAM
	MOVE.W  #0,FCR_1        ; Clear RFCR and TFCR
	MOVE.W  #$0A,MRBLR_1    ; MAX BUFFER length
	MOVE.W  #$4,MIDL_1      ; MAX_IDL
* The following must be issued before a stop transmit command is executed
*       MOVE.W  #$1,BRKCR_1     ; BRKCR = 1.Only one break char. sent if
*                         STOP TRANSMIT command is executed.
* Init counters
	MOVE.W  #$0,PAREC_1
	MOVE.W  #$0,FRMEC_1
	MOVE.W  #$0,NOSEC_1
	MOVE.W  #$0,BRKEC_1
* Put 'a' and 'A' into user defined characters.
	MOVE.W  #$61,ABCR1_1    ; ASCII for 'a'
	MOVE.W  #$41,ABCR2_1    ; ASCII for 'A'
* Set default SCC mode: Uart, 8-char, no parity, 1 stop, s/w operation
	MOVE.W  #$013D,D_SCM_1
*       UART Address characters not used in normal operation.
	MOVE.W  #$8000,RCH1_1
	MOVE.W  #$0000,RCH8_1   ; no software echoing in this example
* No control characters are initialized to cause special interrupts.
* The flow-control facility (XON-XOFF) is not used.
	MOVE.B  #$FF,SCCE1      ; clear SCCE1
	MOVE.B  #$08,SCCM1      ; only interrupt for RX control character

**********************************************************************
*** Prepare Buffer Descriptors ***

* Clear M68000 data registers
	CLR.L   D0
	CLR.L   D1
	CLR.L   D2
	CLR.L   D3
	CLR.L   D4
	CLR.L   D5

* SCC1 Rx Buffer Descriptors Initialization values before execution.

* SCC1 Autobaud Descriptors Initialization values before execution:
* 00700430 0000 0008 0002 0000

	LEA.L   RXBD_71,A0      ; A0 points to autobaud command descriptor
	LEA.L   RCH6_1,A1       ; A1 points to the LOOKUP table for autobaud
	MOVE.W  #$0,ST_BD(A0)   ; Clear all Status Bits
	MOVE.W  #LK_SZ,LN_BD(A0)        ; Initialize Lookup Table Size
	MOVE.L  #LOOKUP,PT_BD(A0)       ; Load pointer to Lookup table
	MOVE.L  #RXCHARS,(A1)   ;Load the pointer to rx characters


*       Special function for Auto baud Mode
* At this time the SCCs are still disabled.
* If autobaud is entered dynamically, one should first do the followings
* before issuing ENTER BAUD HUNT for the SCC that is going to run autobaud:
* (1) STOP TRANSMIT, (2) disable ENT and ENR, and (3) ENTER HUNT MODE.

	MOVE.B  #ENBHM1,CR      ; Enter BAUD HUNT mode
LP1     BTST.B  #0,CR   ; Wait for the command to complete
	BNE.B   LP1
	ORI.W   #ENRON,SCM1     ; For SCC1 ENR=1, the transmitter will be
*       automatically enabled by the microcode once
*       it sets up the UART-autobaud-mode.

* Ready to accept interrupt     *** Set IMR ***
	MOVE.W  #$2000,IMR      ; Allow SCC1 interrupts only.
	MOVE.W  #$2000,SR       ; Unmask interrupt
	JMP     MAIN    ; Go to Main routine for Tx and Rx

**************************** Main Routine *********************************

* Set up BD pointers
MAIN    CLR.L   D5      ; COUNTER FOR OVER-RUN
* The following clears out the external data buffer for received characters
RxReady LEA.L   RXCHARS,A5      ; Load address of receive buffer
	MOVE.W  #$20,D4 ; Load counter for clear loop
CLRLP   MOVE.L  #0,(A5)+        ; Clear out the memory
	DBNE    D4,CLRLP
	LEA.L   RXCHARS,A5      ; Load address of character buffer

*Wait for character to be received.
WAITF   CMPI.W  #0,RXFLAG       ;COMPARE COUNTER        
	BNE     RX_REC
	BTST.B  #1,RXBD_71+1    ; CHECK OV
	BEQ     WAITF
	ADDI.W  #1,D5   ; INCREMENT OVERRUN COUNTER

* This just returns to wait for the next character.
* At this point or in the interrupt handler, the user should put code
* that checks for character length and parity as well as the end of
* message.
*
RX_REC  BRA     WAITF

* When the end of message is detected, the user is responsible for
* the following:
* (1) STOP TRANSMIT, (2) disable ENT and ENR, (3) ENTER HUNT MODE,
* (4) write the Rx and Tx internal states back to their reset values,
* (5) write the Rx and Tx buffer numbers back to their rest values,
* (6) program the SCON register for the correct baudrate (returned at  base + $4A0)
* (7) program the SCC mode to UART mode with the correct character length and parity
* (8) issue the RESTART Transmit and Enter Hunt Mode commands
* and (9) and enable the receiver and transmitter.

***************** SCC1 Interrupt handler ***************************
* This routine only handle the case when RX=1
*
	ORG     INT_VEC ; Interrupt Vector for SCC1
INTSCC1 CLR.L   D1      ; Clear D1
	MOVE.B  SCCE1,D1        ; SCCE1 => D1
	ANDI.W  #8,D1   ; Is CCR set?
	CMPI.W  #0,D1   ; If set
	BNE.B   RXINT1  ; Handle receiver's interrupt
*
* The handling of other events, e.g., CTS, CD IDL, BSY, is left to
* the users as desired.
*
EXINT1  MOVE.W  #$2000,ISR      ; Clear SCC1 bit in ISR
	RTE

************* Receiver portion of SCC1 interrupt routine *************
* This routine handles received (non-empty) BD: set data length = 0,
* clear status bits, set empty = 1, and update PRD
*
* Clear the identified events as soon as possible, so as not to lost
* events occur during the interrupt handling
RXINT1  MOVE.B  #8,SCCE1        ; Clear RX in SCCE1
*
* While Not-Empty continue to process the next Character, Else Exit.
*
NXPRD1  MOVE.L  A5,D1   ; Load current pointer
	CMP.L   TXINTPT.L,D1    ; Check to see if a char was rx
	BEQ     EXINT1  ; If no, then exit
	MOVE.B  (A5)+,D1        ; Load Character
	CMP.B   #CARRT,D1       ; Is the char a carriage return
	BEQ     EXRX1   ; If so, then set flag and exit
	BRA     NXPRD1  ; Otherwise check for another char

EXRX1   MOVE.W  #1,RXFLAG       ; SET SEMAPHORE
	JMP     EXINT1  ; Exit receiver portion of the handler
	
**********************************************************************

	END
